@@ -8,11 +8,15 @@ use std::{
 };
 #[cfg(feature = "terminal-io")]
 use terminal_io::ReadTerminal;
-#[cfg(not(windows))]
-use unsafe_io::os::posish::{AsRawFd, RawFd};
 #[cfg(windows)]
-use unsafe_io::os::windows::{AsRawHandleOrSocket, RawHandleOrSocket};
-use unsafe_io::OwnsRaw;
+use unsafe_io::os::windows::{
+    AsHandleOrSocket, AsRawHandleOrSocket, BorrowedHandleOrSocket, RawHandleOrSocket,
+};
+#[cfg(not(windows))]
+use {
+    io_lifetimes::{AsFd, BorrowedFd},
+    unsafe_io::os::posish::{AsRawFd, RawFd},
+};
 
 /// Adapts an `Read` to implement `ReadLayered`.
 pub struct LayeredReader<Inner> {
@@ -24,7 +28,7 @@ pub struct LayeredReader<Inner> {
 #[cfg(feature = "terminal-io")]
 impl<Inner: ReadTerminal> LayeredReader<Inner> {
     /// Construct a new `LayeredReader` which wraps `inner`, which implements
-    /// `AsUnsafeHandle`, and automatically sets the `line_by_line` setting if
+    /// `ReadTerminal`, and automatically sets the `line_by_line` setting if
     /// appropriate.
     pub fn maybe_terminal(inner: Inner) -> Self {
         let line_by_line = inner.is_line_by_line();
@@ -236,7 +240,7 @@ impl<RW: terminal_io::ReadTerminal> terminal_io::ReadTerminal for LayeredReader<
 }
 
 #[cfg(not(windows))]
-impl<Inner: Read + AsRawFd + OwnsRaw> AsRawFd for LayeredReader<Inner> {
+impl<Inner: Read + AsRawFd> AsRawFd for LayeredReader<Inner> {
     #[inline]
     fn as_raw_fd(&self) -> RawFd {
         match &self.inner {
@@ -246,8 +250,19 @@ impl<Inner: Read + AsRawFd + OwnsRaw> AsRawFd for LayeredReader<Inner> {
     }
 }
 
+#[cfg(not(windows))]
+impl<Inner: Read + AsFd> AsFd for LayeredReader<Inner> {
+    #[inline]
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        match &self.inner {
+            Some(inner) => inner.as_fd(),
+            None => panic!("as_fd() called on closed LayeredReader"),
+        }
+    }
+}
+
 #[cfg(windows)]
-impl<Inner: Read + AsRawHandleOrSocket + OwnsRaw> AsRawHandleOrSocket for LayeredReader<Inner> {
+impl<Inner: Read + AsRawHandleOrSocket> AsRawHandleOrSocket for LayeredReader<Inner> {
     #[inline]
     fn as_raw_handle_or_socket(&self) -> RawHandleOrSocket {
         match &self.inner {
@@ -257,8 +272,16 @@ impl<Inner: Read + AsRawHandleOrSocket + OwnsRaw> AsRawHandleOrSocket for Layere
     }
 }
 
-// Safety: `LayeredReader` implements `OwnsRaw` if `Inner` does.
-unsafe impl<Inner: Read + OwnsRaw> OwnsRaw for LayeredReader<Inner> {}
+#[cfg(windows)]
+impl<Inner: Read + AsHandleOrSocket> AsHandleOrSocket for LayeredReader<Inner> {
+    #[inline]
+    fn as_handle_or_socket(&self) -> BorrowedHandleOrSocket<'_> {
+        match &self.inner {
+            Some(inner) => inner.as_handle_or_socket(),
+            None => panic!("as_handle_or_socket() called on closed LayeredReader"),
+        }
+    }
+}
 
 impl<Inner: fmt::Debug> fmt::Debug for LayeredReader<Inner> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
