@@ -3,11 +3,15 @@ use std::{
     fmt::{self, Arguments},
     io::{self, IoSlice, Write},
 };
-#[cfg(not(windows))]
-use unsafe_io::os::posish::{AsRawFd, RawFd};
 #[cfg(windows)]
-use unsafe_io::os::windows::{AsRawHandleOrSocket, RawHandleOrSocket};
-use unsafe_io::OwnsRaw;
+use unsafe_io::os::windows::{
+    AsHandleOrSocket, AsRawHandleOrSocket, BorrowedHandleOrSocket, RawHandleOrSocket,
+};
+#[cfg(not(windows))]
+use {
+    io_lifetimes::{AsFd, BorrowedFd},
+    unsafe_io::os::posish::{AsRawFd, RawFd},
+};
 
 /// Adapts a [`std::io::Write`] to implement [`WriteLayered`].
 pub struct LayeredWriter<Inner> {
@@ -184,6 +188,17 @@ impl<Inner: Write + AsRawFd> AsRawFd for LayeredWriter<Inner> {
     }
 }
 
+#[cfg(not(windows))]
+impl<Inner: Write + AsFd> AsFd for LayeredWriter<Inner> {
+    #[inline]
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        match &self.inner {
+            Some(inner) => inner.as_fd(),
+            None => panic!("as_fd() called on closed LayeredWriter"),
+        }
+    }
+}
+
 #[cfg(windows)]
 impl<Inner: Write + AsRawHandleOrSocket> AsRawHandleOrSocket for LayeredWriter<Inner> {
     #[inline]
@@ -195,8 +210,16 @@ impl<Inner: Write + AsRawHandleOrSocket> AsRawHandleOrSocket for LayeredWriter<I
     }
 }
 
-// Safety: `LayeredWriter` implements `OwnsRaw` if `Inner` does.
-unsafe impl<Inner: Write + OwnsRaw> OwnsRaw for LayeredWriter<Inner> {}
+#[cfg(windows)]
+impl<Inner: Write + AsHandleOrSocket> AsHandleOrSocket for LayeredWriter<Inner> {
+    #[inline]
+    fn as_handle_or_socket(&self) -> BorrowedHandleOrSocket<'_> {
+        match &self.inner {
+            Some(inner) => inner.as_handle_or_socket(),
+            None => panic!("as_handle_or_socket() called on closed LayeredWriter"),
+        }
+    }
+}
 
 impl<Inner: fmt::Debug> fmt::Debug for LayeredWriter<Inner> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
